#ident "@(#) $Id: ordwrite.c,v 1.1 2002/06/18 20:37:21 bzfkocht Exp $"
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                           */
/*   File....: ordwrite.c                                                    */
/*   Name....: ORD File Write                                                */
/*   Author..: Thorsten Koch                                                 */
/*   Copyright by Author, All rights reserved                                */
/*                                                                           */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*
 * Copyright (C) 2001 by Thorsten Koch <koch@zib.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include "mshell.h"
#include "portab.h"
#include "mme.h"
#include "rbt.h"
#include "lpstore.h"

/* A specification for the ORD file format can be found in the
 * ILOG CPLEX 7.0 Reference Manual page 545.
 */
void ord_write(
   const Lps* lp,
   FILE*      fp)
{
   const Var*  var;
   char  vtmp  [13];
   
   assert(lp != NULL);
   assert(fp != NULL);

   fprintf(fp, "* This file was automatically generated by zimpl\n");
   fprintf(fp, "NAME        %8.8s\n", lp->name);

   for(var = lp->var_root; var != NULL; var = var->next)
   {
      if (var->type == VAR_CON)
         continue;
      
      lps_makename(vtmp, MPS_NAME_LEN + 1, var->name, var->number);

      fprintf(fp, "    %-8.8s  %8d", vtmp, var->priority);

      if (LT(var->startval, INFINITY))
         fprintf(fp, "  %.10e", var->startval);

      fputc('\n', fp);
   }
   fprintf(fp, "ENDATA\n");
}   


